# BrowserGym: a Gym Environment for Web Task Automation

[[Setup]](#setup) ♦ [[Usage]](#usage) ♦ [[Demo]](#demo)

This package provides `browsergym`, a gym environment for web task automation in the Chromium browser.

https://github.com/ServiceNow/BrowserGym/assets/26232819/e0bfc788-cc8e-44f1-b8c3-0d1114108b85

_Example of a GPT4-V agent executing openended tasks (top row, chat interactive), as well as WebArena and WorkArena tasks (bottom row)_

BrowserGym includes the following benchmarks by default:
 - [MiniWob++](https://miniwob.farama.org/)
 - [WebArena](https://webarena.dev/)
 - [WorkArena](https://github.com/ServiceNow/WorkArena)

Designing new web benchmarks with BrowserGym is easy, and simply requires to inherit the [`AbstractBrowserTask`](https://github.com/ServiceNow/BrowserGym/blob/main/core/src/browsergym/core/task.py#L7C7-L7C26) class.

## Setup

To install browsergym, you can either install one of the `browsergym-miniwob`, `browsergym-webarena` and `browsergym-workarena` packages, or you can simply install `browsergym` which includes all of these by default.
```sh
pip install browsergym
```

Then, a required step is to setup playwright by running
```sh
playwright install
```

Finally, each benchmark comes with its own specific setup that requires to follow additional steps.
 - for miniwob, see [miniwob/README.md](miniwob/README.md)
 - for webarena, see [webarena/README.md](webarena/README.md)
 - for workarena, see [WorkArena](https://github.com/ServiceNow/WorkArena)


# WebArena

## Baselines
### 1. Raw observation
```
python webarena_src/webarena_eval_baseline.py --backbone <backbone name> --max_steps 20
```

### 2. Self-contextualization
```
python webarena_src/webarena_eval_selfctx.py \
    --backbone <backbone name> \
    --max_steps 20
```


## LCoW
### 1. Sampling contextualized observations
```
python webarena_src/contextualization_sampling_init.py
```
### 2. Train contextualization module
```
python webarena_src/aggregate_data_init.py
python webarena_src/train.py --iter 0
```
### 3. Evaluation
```
python webarena_src/webarena_eval_lcow.py \
    --backbone <backbone name> \
    --max_steps 20 \
    --ckpt_step 379
```


# WorkArena
## Baselines
### 1. Raw observation
```
python workarena_src/workarena_eval_baseline.py --backbone <backbone name> --max_steps 20
```

### 2. Self-contextualization
```
python workarena_src/workarena_eval_selfctx.py --backbone <backbone name> --max_steps 20
```


## LCoW
### 1. Sampling contextualized observations
```
python workarena_src/contextualization_sampling_init.py
```
### 2. Train contextualization module
```
python workarena_src/aggregate_data_init.py
python workarena_src/train.py --iter 0
```
### 3. Evaluation
```
python workarena_src/workarena_eval_lcow.py \
    --backbone <backbone name> \
    --max_steps 20 \
    --ckpt_step <ckpt_step>
```

