# WebShop Experiment

## 1. Setup
* Refer to the setup code for AgentLumos (https://github.com/allenai/lumos).
* Additionally, you have to login to wandb.

## 2. Quick Walkthrough for Webshop
```python
import gym
from web_agent_site.envs import WebAgentTextEnv
from web_agent_site.utils import DEBUG_PROD_SIZE

# intialize environment
env = gym.make('WebAgentTextEnv-v0', observation_mode='text_rich', num_products=DEBUG_PROD_SIZE)

# reset environment (each task idx corresponds to different human instruction)
task_idx = 0 # (0 ~ 6510)
obs, info = env.reset(session=task_idx)
print(obs)
action = 'search[iphone 13 pro]'
# reward is given at the end of the episode (when you click[buy now]), based on the task information.
obs, reward, done, info = env.step(action)
print(obs)
```

## 3. Experiment 

```
cd webshop
export GEMINI_API_KEY = 'your gemini api key'
export OPENAI_API_KEY = 'your openai api key'
export ANTHROPIC_API_KEY = 'your anthorpic api key'
export LLAMA_API_KEY = 'your llama api key (deepinfra.ai)'
```

### Evaluating baseline
* base prompt
```
python src/webshop_baseline_eval.py --num_tasks 500 --backbone [e.g., openai/gpt-4o-2024-08-06, googleai/gemini-1.5-flash-002]
```

* self-contextualization
```
python src/webshop_baseline_eval.py --num_tasks 500 --backbone [e.g., openai/gpt-4o-2024-08-06, googleai/gemini-1.5-flash-002] --rephrase 
```

### LCoW (iteraion = 1)

* Sampling contextualized observations
```
python src/collect_data.py --backbone googleai/gemini-1.5-flash-002 --num_samples 4
```

* Training contextualization module
```
python src/train.py --iter 0
```

* Evaluation
```
python src/webshop_lcow_eval.py --num_tasks 500 --iter 0 --ckpt_step 265 --backbone [googleai/gemini-1.5-flash-002]
```

### LCoW (iteraion > 1)

* Collecting successful trajectories
```
python src/traj_collection.py --backbone googleai/gemini-1.5-flash-002 --iter [iteration] --ckpt_step [ckpt_step]
```

* Sampling contextualized observations
```
python src/collect_data_iter.py --iter [iteration] --num_samples 4 --ckpt_step [ckpt_step]
```

* Training contextualization module
```
python src/train.py --iter [iteration]
```

* Evaluation
```
python src/webshop_lcow_eval.py --num_tasks 500 --iter [iteration] --ckpt_step [ckpt_step] --backbone [googleai/gemini-1.5-flash-002]
```


